#!/usr/bin/env node

'use strict'

const path = require('path'),
      minimist = require('minimist'),
      mkdirp = require('mkdirp'),

      splitCss = require('extract-css-media'),

      files = require('@mightyplow/jslib').file,
      html = require('@mightyplow/jslib').html,
      string = require('@mightyplow/jslib').string

const normalizeString = filename => filename.replace(/[^0-9a-z._]+/gi, '_').replace(/^_?(.*?)_?$/, '$1')
const prefixString = prefix => str => prefix + str

// get command line arguments
const args = minimist(process.argv.slice(2)),
    inputFilename = args.inputFilename

// exit if input filename is not set
if (!inputFilename) {
    console.log('No input file specified.')
    console.log('Use --inputFilename=[filename] to specify an input css file.\n')
    process.exit(1)
}

const options = {
    inputFilename: inputFilename,
    outDir: args.outDir || './',
    minRules: Number(args.minRules) || 20,
    compress: (args.compress !== undefined) ? string.toBool(args.compress) : true,
    normalizeFilenames: (args.normalizeFilenames !== undefined) ? string.toBool(args.normalizeFilenames) : true,
    showLinkTags: (args.showLinkTags !== undefined) ? string.toBool(args.showLinkTags) : false
}

const writeFileSuccessHandler = (filename, selector) => () => {
    console.log(`${filename} created.`)

    return {
        filename,
        selector
    }
}

files.readFile(options.inputFilename).then(data => {
    const filteredBundles = splitCss(data.toString(), {
        compress: options.compress,
        minRules: options.minRules,
        source: options.inputFilename
    })

    const inputFileExt = path.extname(options.inputFilename)
    const inputBase = path.basename(inputFilename, inputFileExt)
    const prefixFilename = prefixString(string.append(inputBase, '_'))
    const prefixOutDir = path.join.bind(null, options.outDir)

    mkdirp(options.outDir, (err, result) => {
        if (err) {
            throw Error('Error while creating output directory');
        }

        // create css files for the bundles and return a promise for each
        const writeFiles = Object.keys(filteredBundles).map(mediaSelector => {
            const filenameBase = prefixFilename(mediaSelector)
            const normalizedFilenameBase = options.normalizeFilenames ? normalizeString(filenameBase) : filenameBase
            const filename = prefixOutDir(string.append(normalizedFilenameBase, '.css'))

            return files.writeFile(filename, filteredBundles[mediaSelector]).then(writeFileSuccessHandler(filename, mediaSelector))
        })

        return Promise.all(writeFiles).then(results => {
            if (options.showLinkTags) {
                // empty line to separate the creation messages from the tags
                console.log()

                results.forEach(result => {
                    console.log(html.createLinkTag(result.filename, result.selector))
                })
            }

            console.log('\ndone.')
        })
    })
}).catch(err => {
    console.log(err)
})